@@ -7,13 +7,14 @@
 #'   The density is normalized crudely using the trapezium rule.  For
 #'   \code{d = 2} a scatter plot of the simulated values is produced with
 #'   density contours superimposed.  For \code{d > 2} pairwise plots of the
-#'   simulated values are produced using \code{pairs()}.
+#'   simulated values are produced.
 #'
 #' @param x an object of class "ru", a result of a call to \code{ru}.
 #' @param y Not used.
 #' @param ... Additional arguments passed on to \code{hist}, \code{lines},
 #'   \code{contour} or \code{points}.
-#' @param n A number.  The meaning depends on the value of x$d.
+#' @param n A numeric scalar.  Only relevant if \code{x$d = 1} or
+#'   \code{x$d = 2}. The meaning depends on the value of x$d.
 #' \itemize{
 #'   \item {For d = 1 : n + 1 is the number of abscissae in the trapezium
 #'      method used to normalize the density.}
@@ -29,12 +30,11 @@
 #'   rows of plots.  If the user doesn't provide this then it is set
 #'   internally.
 #' @param xlabs,ylabs Numeric vectors.  When \code{d} > 2 these set the labels
-#'   on the x and y axes respectively.  if the use doesn't provide these then
+#'   on the x and y axes respectively.  If the user doesn't provide these then
 #'   the column names of the simulated data matrix to be plotted are used.
-#' @details
-#' Note that \code{suppressWarnings} is used to avoid potential benign warnings
-#'   caused by passing unused graphical parameters to \code{hist} and
-#'   \code{lines} via \code{...}.
+#' @param points_par A list of arguments to pass to
+#'   \code{\link[graphics]{points}} to control the appearance of points
+#'   depicting the simulated values. Only relevant when \code{d = 2}.
 #' @examples
 #' # Log-normal density ----------------
 #' x <- ru(logf = dlnorm, log = TRUE, d = 1, n = 1000, lower = 0, init = 1)
@@ -67,7 +67,8 @@
 #' @export
 plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
                     prob = c(0.1, 0.25, 0.5, 0.75, 0.95, 0.99),
-                    ru_scale = FALSE, rows = NULL, xlabs = NULL, ylabs = NULL) {
+                    ru_scale = FALSE, rows = NULL, xlabs = NULL,
+                    ylabs = NULL, points_par = list(col = 8)) {
   if (!inherits(x, "ru")) {
     stop("use only with \"ru\" objects")
   }
@@ -79,17 +80,26 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
     plot_density <- x$logf_rho
   } else {
     plot_data <- x$sim_vals
-    plot_density <- x$logf
+    pmedian <- apply(x$sim_vals, 2, stats::median)
+    hshift <- do.call(x$logf, c(list(pmedian), x$logf_args))
+    xlogf <- x$logf
+    plot_density <- function(x, ...) {
+      xlogf(x, ...) - hshift
+    }
+  }
+  if (!is.null(x$logf_rho_args) & ru_scale) {
+    density_args <- x$logf_rho_args
+  } else {
+    density_args <- x$logf_args
   }
   if (x$d == 1) {
-    temp <- suppressWarnings(graphics::hist(plot_data, prob = TRUE,
-                                            plot = FALSE))
+    temp <- graphics::hist(plot_data, plot = FALSE)
     a <- temp$breaks[1]
     b <- temp$breaks[length(temp$breaks)]
     h <- (b-a)/n
     xx <- seq(a, b, by = h)
     density_fun <- function(z) {
-      density_list <- c(list(z), x$logf_args)
+      density_list <- c(list(z), density_args)
       exp(do.call(plot_density, density_list))
     }
     yy <- sapply(xx, density_fun)
@@ -103,17 +113,23 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
     yy <- yy / area
     max_y <- max(temp$density, yy)
     temp <- list(...)
+    my_hist <- function(x, ..., type, lty, lwd, pch, lend, ljoin, lmitre) {
+      graphics::hist(x, ...)
+    }
     if (is.null(temp$xlab)) {
-      suppressWarnings(graphics::hist(plot_data, prob = TRUE, main="",
-                                      ylim = c(0, max_y), xlab = "", ...))
+      my_hist(plot_data, prob = TRUE, main = "", ylim = c(0, max_y), xlab = "",
+              ...)
       if (!is.null(colnames(plot_data))) {
         graphics::title(xlab = parse(text = colnames(plot_data)[1]))
       }
     } else {
-      suppressWarnings(graphics::hist(plot_data, prob = TRUE, main="",
-                                      ylim = c(0, max_y), ...))
+      my_hist(plot_data, prob = TRUE, main = "", ylim = c(0, max_y), ...)
     }
-    suppressWarnings(graphics::lines(xx, yy, ...))
+    my_lines <- function(x, y, ..., breaks, freq, probability, include.lowest,
+                         right, density, angle, border, plot, labels, nclass) {
+      graphics::lines(x, y, ...)
+    }
+    my_lines(xx, yy, ...)
   }
   if (x$d == 2) {
     r <- apply(plot_data, 2, range)
@@ -123,7 +139,7 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
     zz <- matrix(NA, ncol = length(xx), nrow = length(yy))
     for (i in 1:length(xx)) {
       for (j in 1:length(yy)) {
-        for_logf <- c(list(c(xx[i], yy[j])), x$logf_args)
+        for_logf <- c(list(c(xx[i], yy[j])), density_args)
         zz[i, j] <- exp(do.call(plot_density, for_logf))
       }
     }
@@ -134,7 +150,7 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
     #
     graphics::contour(xx, yy, zz, levels = con.levs, add = F, ann = F,
       labels = prob * 100, ...)
-    graphics::points(plot_data, col = 8, ...)
+    do.call(graphics::points, c(list(x = plot_data), points_par))
     graphics::contour(xx, yy, zz, levels = con.levs, add = T, ann = T,
       labels = prob * 100, ...)
     temp <- list(...)
@@ -154,12 +170,19 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
       rows <- x$d -2
     }
     cols <- ceiling(choose(x$d, 2) / rows)
-    temp <- list(...)
     if (is.null(xlabs)) {
-      xlabs <- colnames(plot_data)
+      if (!is.null(colnames(plot_data))) {
+        xlabs <- colnames(plot_data)
+      } else {
+        xlabs <- rep(NA, x$d)
+      }
     }
     if (is.null(ylabs)) {
-      ylabs <- colnames(plot_data)
+      if (!is.null(colnames(plot_data))) {
+        ylabs <- colnames(plot_data)
+      } else {
+        ylabs <- rep(NA, x$d)
+      }
     }
     def.par <- graphics::par(no.readonly = TRUE)
     graphics::par(mfrow = c(rows, cols))
@@ -211,8 +234,7 @@ plot.ru <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
 #' summary(x)
 #' @seealso \code{\link{ru}} for descriptions of \code{object$sim_vals} and
 #'   \code{object$box}.
-#' @seealso \code{\link{plot.ru}} for a diagnostic plot (for \code{d} = 1
-#'   and \code{d} = 2 only).
+#' @seealso \code{\link{plot.ru}} for a diagnostic plot.
 #' @export
 summary.ru <- function(object, ...) {
   if (!inherits(object, "ru")) {
